#pragma TextEncoding	= "UTF-8"
#pragma rtGlobals 		= 3
#pragma IgorVersion		= 8.0
#pragma ModuleName		= BackgroundRemover
#pragma version			= 1.56
#pragma DefaultTab		= {3,20,4}		// set default tab width in Igor Pro 9 and later

static Constant kVersion = 1.56
static StrConstant kVersionDate = "04/2025"

// --------------------- Project Updater header ----------------------
// If you're using Igor Pro 8 or later and have Project Updater
// installed, this package can check periodically for new releases.
// https://www.wavemetrics.com/project/Updater
static Constant kProjectID = 21532 																// the project node on IgorExchange
static StrConstant ksShortTitle = "Background Remover"											// the project short title on IgorExchange

//________________________________________________________________________________________________
//	Written by Stephan Thuermer - https://www.wavemetrics.com/user/chozo
//	Can be used to remove the "background" from spectra using various models.
//	
//	Constant:		A constant value.
//	Step:			Smooth step function which connects the two endpoints (using tanh()).
//	Linear:			Linear function through the two endpoints.
//	Exponential:	Normalized exponential function connecting the two endpoints.
//	Double Exp.:	Double exponential function fitted within the selected range.
//	Polynomial:		Polynomial function fitted within the selected range.
//	Total Sum:		Total sum inelastic scattering background after X. Li et al., J.Electron Spectrosc. Relat. Phenom. 63 (1993) 253-265
//	Shirley:		Iterative Shirley inelastic scattering background after A. Proctor and P. A. Sherwood, Anal. Chem. 54, 13-19 (1982)
//	Tougaard:		Three- and four-parameter universal scattering background after S. Tougaard, Surf. Interface Anal. 25: 137-154 (1997)
//	Tougaard Data:	Scattering background using scattering cross-section data after S. Tougaard, Surf. Interface Anal. 25: 137-154 (1997)
//
//	You can generate backgrounds in user code using the function:
//		string result_message = GenerateBackground(backstruct, datawave, backwave, [from, to])
//	
//	Populate the BackRemoveStruct with the desired parameters and provide both the original data (datawave) and a wave which
//	the background will be written to (backwave); both datawave and backwave need to have the same size and scaling.
//	The 'from' and 'to' optional parameters are for setting the start and end positions of the background calculation.
//	If these parameters are omitted then the first and last points of the input data are used (does not work with poly background).
//________________________________________________________________________________________________

//________________________________________________________________________________________________
// 2021-01-23 - ver. 1.30:	Initial public release.
// 2021-01-25 - ver. 1.31:	Fixed error when DataBroswer is not open & minor control size adjustments.
// 2021-02-04 - ver. 1.32:	Minor window and control size adjustments for an improved layout on Mac.
// 2021-05-16 - ver. 1.40:	Minor code refactoring.
//							Now background function settings are saved into the output's notes.
//							If multiple backgrounds are subtracted then each setting is recorded successively.
// 2021-05-17 - ver. 1.41:	The polynomial coefficients are now displayed in the legend and saved in the output.
//							Fixed several bugs with the polynomial background when called from user code without the GUI.
//							 - error because folder was not present.
//							 - error when selected range is close to the full data range.
//							 - error when polynomial degree is invalid.
// 2022-03-16 - ver. 1.50:	Fixed error on bad input to BackgroundSubtractGUI().
//							Previous settings are now loaded from saved background waves (ending "_bck").
//							Added setting for Tougaard band-gap (T0) smoothing and a link setting between parameters D and C.
//							Generated scattering CS data from the parametric Tougaard type will be exported as well (ending "_scs").
//							Output waves are now saved in the same folder as the input.
//							Smoothing of the step function can now be set in a wider range.
//							Now multiple instances of the tool can be opened at the same time.
//							Parameters will be written into background waves when calling GenerateBackground() in user code as well.
//							Rearranged control layout and added 'Plot after Export' function to create a graph of exported background data.
//							Added support for keyboard-shortcut (and other future) settings via external text file.
// 2022-07-14 - ver. 1.51:	Re-positioned some controls for consistency.
//							Follow Data checkbox has been added to the exponential and polynomial functions.
//							Minor bug-fixes and code improvements.
// 2023-03-19 - ver. 1.55:	Added double exponential function.
//							Rearranged controls for clarity.
// 2025-04-22 - ver. 1.56:	Upgraded to Igor 8 code.
//							Fixed bug: Debugger was not switched off for background calculations.
//________________________________________________________________________________________________


// *** global definitions ************************************************************************
StrConstant kBackgroundTypes = "Constant;Step;Linear;Exponential;Double Exponential;Polynomial;Total Sum;Shirley;Tougaard;Tougaard Data;"		// the various types of backgrounds used - non-static for backwards compatibility
static StrConstant kWorkingDir = "Background_Remover"											// the name for the working dir
static StrConstant kSettingsFileName = "Spectra Tools settings.dat"								// just for keyboard shortcuts for now.
static StrConstant kTougaardIniPara  = "2866;1643;0;0;"											// initial parameters B,C,D and T0 for the universal Tougaard background (common metal)
//static StrConstant kTougaardIniPara= "325;542;275;5;"											// initial parameters B,C,D and T0 for SiO2
static Constant kTougaardSmoothStep = 0.5														// !only used if not set in the structure! - how much the insulator step for parameter T0 will be smoothed (should not be 0)
static Constant kMaxIterations		= 50														// iteration limit for Shirley background calculation
static Constant kConvergelimit		= 1e-6														// converge limit for Shirley background calculation

Menu "Spectra Tools"
	BackgroundRemover#MenuEntry(),/Q, CreateBrowser; BackgroundSubtractGUI($GetBrowserSelection(0))
End


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static function/S MenuEntry()																	// read external settings if available
	int fileID;	string read = ""
	Open/Z/R fileID as ParseFilePath(1, FunctionPath(""), ":", 1, 0)+kSettingsFileName
	if (!V_flag)
		do
			FReadLine fileID, read
		while (strlen(read) > 0 && !StringMatch(read, "start background remover*"))
		read = ReplaceString("\t", ReplaceString(" ", read, ""), "")
		read = StringByKey(ReplaceString(" ", "start background remover", ""),read,"=","\r")
		Close fileID
	endif
	return "Background Subtraction ..."+SelectString(strlen(read),"","/"+read)
end

// *** BR kernel structure *** ###################################################################

Structure BackRemoveStruct
	string FuncType		// Constant, Step, Linear, Exponential, Polynomial, Total Sum, Shirley, Tougaard, Tougaard Data
	variable avgPercent	// y Values around range ends positions get averaged
	// function specific:
	variable expScale	// Exponential / Double Exponential
	variable polyDegree	// Poly
	variable fullLinear	// Line
	// for Step
	variable stepPos	// x center position
	variable stepSmooth	// smoothing value
	// for Tougaard (universal CS parameters)
	variable TouB
	variable TouC
	variable TouD
	variable TouT
	variable TouSmooth	// how much the insulator step for parameter T0 will be smoothed (should not be 0)
	// Tougaard Data
	variable CSscale	// scaling (default 1)
	string CSdata		// name of CS wave in current folder
	// Shirley and Total Sum
	variable followData	// follow data outside range
	variable smoothData	// integer between 3 and 20
EndStructure

function BackRemoveInitialize(bs)
	STRUCT BackRemoveStruct &bs
	bs.FuncType		= "Constant"
	bs.avgPercent	= 0.5
	bs.expScale		= 1
	bs.polyDegree	= 2
	bs.fullLinear	= 0
	bs.stepPos		= 0
	bs.stepSmooth	= 1
	bs.TouB			= str2num(StringFromList(0,kTougaardIniPara))
	bs.TouC			= str2num(StringFromList(1,kTougaardIniPara))
	bs.TouD			= str2num(StringFromList(2,kTougaardIniPara))
	bs.TouT			= str2num(StringFromList(3,kTougaardIniPara))
	bs.TouSmooth	= 0.5
	bs.CSscale		= 1
	bs.CSdata		= "none"
	bs.followData	= 0
	bs.smoothData	= 5
	return 0
end

// *** BR kernel (both for UI and user code) *** ##################################################

function/S GenerateBackground(STRUCT BackRemoveStruct &s, wave inwave, wave backwave, [variable from, variable to])
	if (!WaveExists(inwave) || !WaveExists(backwave))
		return "Input waves missing."
	endif
	if (WaveType(inwave,1) != 1 || WaveDims(inwave) > 1 || WaveType(backwave,1) != 1 || WaveDims(backwave) > 1)
		return "This tool works only for numeric 1D waves."
	endif
	if (DimSize(inwave,0) != DimSize(backwave,0))
		return "Input waves need to have the same size and scaling."
	endif
	
	variable rows = DimSize(inwave,0), delta = DimDelta(inwave,0)
	variable P1 = ParamIsDefault(from) ? 0      : x2pnt(inwave,from)
	variable P2 = ParamIsDefault(to)   ? rows-1 : x2pnt(inwave,to)
	from = ParamIsDefault(from) ? pnt2x(inwave,0)      : from
	to   = ParamIsDefault(to)   ? pnt2x(inwave,rows-1) : to
	variable xRange	= s.avgPercent/100*abs(rightx(inwave)-leftx(inwave))						// define the averaging range
	variable Up		= mean(inwave, from-xRange, from+xRange)									// upper y value to connect to
	variable Down	= mean(inwave, to  -xRange, to  +xRange)									// lower y value
	
	if (P1 < 0 || P2 > (rows-1))
		return "Range input is outside data's x scaling."
	endif

	DFREF saveDF = GetDataFolderDFR()
	
	backwave[,P1] = Up																			// upper constant part of background	
	backwave[P2,] = Down																		// lower constant part of background
	
	DebuggerOptions; int sav_debug = V_debugOnError
	DebuggerOptions debugOnError=0 																// switch this off in case the fit fails
	
	string message = "", notestr = ""															// output message and wave notes
	int i = 0, cferror = 0
	Strswitch(s.FuncType)
		case "Constant":
			backwave = Down
		break
		case "Step":
			backwave = Down + (Up-Down)/2*(1-tanh(sign(delta)*(x-s.stepPos)/s.stepSmooth))		// tanh used to generate a smooth transition
			notestr += "\rstep position = "+num2str(s.stepPos)
			notestr += "\rstep smoothing = "+num2str(s.stepSmooth)
		break
		case "Linear":
			if (s.fullLinear)
				backwave[0,rows-1] = Up-(Up-Down)*(p-P1)/(P2-P1)
			else
				backwave[P1,P2]    = Up-(Up-Down)*(p-P1)/(P2-P1)
			endif
			notestr += "\rfull line = "+SelectString(s.fullLinear,"no","yes")
		break
		case "Exponential":
			backwave = s.expScale*(Up-Down)*(exp((P2-p)/(P2))-1)/(exp((P2-P1)/(P2))-1) + Down	// normalized exponential (is 1 at P1 and 0 at P2) which gets shifted and scaled
			notestr += "\rfollow data = "+SelectString(s.followData,"no","yes")
			notestr += "\rexp. scale = "+num2str(s.expScale)
		break
		case "Polynomial":
			if (s.polyDegree < 3 || s.polyDegree > 20)
				message += "Polynomial order outside valid range of 3 to 20."
				break
			endif
			if (P2-P1+s.polyDegree >= rows)
				if (rows > 50)																	// slightly adjust the range to make the fit work
					P1 += s.polyDegree
					P2 -= s.polyDegree
					message += "Increased data range by "+num2str(s.polyDegree*2)+" points. "
				else
					message += "Selected range incompatible with polynomial background.\rChoose proper start and end points within the data range."
					break
				endif
			endif
			
			SetDataFolder NewFreeDataFolder()
				Duplicate/Free inwave, FitWave, Mask
				Mask = 1; Mask[P1,P2] = 0														// the mask wave decides the fit range
				backwave = NaN
				
				notestr += "\rfollow data = "+SelectString(s.followData,"no","yes")
				notestr += "\rpoly. degree = "+num2str(s.polyDegree)
				
				try
					CurveFit/X=1/Q/L=(rows) poly s.polyDegree, FitWave /M=Mask; AbortOnRTE
					wave result = W_coef, psigma = W_sigma
					backwave = poly(result,x)
					
					message += "Poly: "	+ num2str(result[0])									// output message writes all coefficients
					for (i = 1; i < numpnts(result);i++)
						message += " "+SelectString(result[i]<0,"+","")+ num2str(result[i]) + "*x" + SelectString(i>1,"","^"+num2str(i))
					endfor
					for (i = 0; i < s.polyDegree; i++)
						notestr += "\rC"+num2str(i)+" = "+num2str(result[i])+" ± "+num2str(psigma[i])
					endfor
				catch
					if (V_AbortCode == -4)
						cfError = GetRTError(1)
						message += "Fit failed!"; noteStr += message
					endif
				endtry
			SetDataFolder saveDF
		break
		case "Double Exponential":
			SetDataFolder NewFreeDataFolder()
				Duplicate/Free inwave, FitWave, Mask
				if (sign(delta) == -1)
					SetScale/P x, -Dimoffset(inwave,0), -delta, FitWave
				endif
				Mask = 1; Mask[P1,P2] = 0
				backwave = NaN
				
				try
					CurveFit/X=1/Q/L=(rows) dblexp_XOffset FitWave /M=Mask; AbortOnRTE
					wave result = W_coef, wX = W_fitConstants, psigma = W_sigma
					backwave = (1-s.expScale)*Down+ s.expScale*(result[0]+result[1]*exp(-(sign(delta)*x-wX[0])/result[2])+result[3]*exp(-(sign(delta)*x-wX[0])/result[4]))
					sprintf message, "y = %g + %g * exp( -(x-%g) / %g ) + %g * exp( -(x-%g) / %g )", result[0],result[1],wX[0],result[2],result[3],wX[0],result[4]
				catch
					if (V_AbortCode == -4)
						cfError = GetRTError(1)
						message += "Fit failed!"
					endif
				endtry
			SetDataFolder saveDF
			
			notestr += "\r"+message
			notestr += "\rexp. scale = "+num2str(s.expScale)
			notestr += "\rfollow data = "+SelectString(s.followData,"no","yes")
			message[0] = "Dbl Exp: "
		break
		case "Total Sum":
			Duplicate/Free inwave, temp; temp -= Down											// clean the data from the constant background first
			backwave[P1,P2] = Down+(Up-Down)*sum(temp,pnt2x(backwave,p),to)/sum(temp,from,to)	// Total background calculation
			
			notestr += "\rfollow data = "+SelectString(s.followData,"no","yes")
			notestr += "\rsmoothing = "+num2str(s.smoothData)
		break
		case "Shirley":
			Duplicate/Free backwave PrevShirley, Difference;	PrevShirley = Down
			do
				Difference = inwave - PrevShirley												// data minus constant background and previous Shirley iteration
				MatrixOP/O Difference = replaceNaNs(Difference,0)								// make sure there are no NaNs which disturb the sum later
				variable KFactor = abs(Up-Down) / area(Difference,from,to)						// Shirley scale factor
				backwave[P1,P2] = KFactor*area(Difference,pnt2x(backwave,p),to)	+ Down			// Shirley background calculation
				MatrixOP/free Rest = abs(sum(PrevShirley - backwave))							// difference between the backgrounds before and after the current iteration
				i++
				if (i >= kMaxIterations)
					message +=  "Shirley calculation not converged. "
					backwave = NaN
					break
				endif
				PrevShirley = backwave															// save for next iteration
			while (Rest[0] > kConvergelimit*Up && i < kMaxIterations)
			message += "Iterations: "+ num2str(i)
			notestr += "\rfollow data = "+SelectString(s.followData,"no","yes")
			notestr += "\rsmoothing = "+num2str(s.smoothData)
		break
		case "Tougaard":
			DFREF inDir = GetWavesDataFolderDFR(inwave)
			Duplicate/O inwave, inDir:$(NameOfWave(inwave)+"_CS")								// prepare CS data wave
			wave CS_Data = inDir:$(NameOfWave(inwave)+"_CS")
			SetScale/P x 0, abs(delta), CS_Data;	CS_Data = 0									// cross section data needs to be properly scaled from zero
			
			variable Toutype = s.TouD > 0 ? -1 : 1												// see if parameter D is set; if yes the formula has to be changed in the denominator to minus
			CS_Data = (s.TouB * x) / ((s.TouC + Toutype * x^2)^2 + s.TouD * x^2)				// universal loss function
			if (numtype(s.TouSmooth) == 0 && s.TouSmooth > 0)
				CS_Data *= 0.5*(1+tanh((x-s.TouT)/s.TouSmooth))									// include band-gap parameter for insulators
			else
				CS_Data *= 0.5*(1+tanh((x-s.TouT)/kTougaardSmoothStep))							// default setting
			endif
			
			Duplicate/Free backwave Scatter;		Scatter = 0									// the scattering share per point
			backwave = 0																		// start with blank background
			for (i = 0; i < rows; i++)
				Scatter[0,rows-i-1] = CS_Data[p] * abs(delta) * (inwave[p+i]-Down)				// calculate the scattering from the current point on
				backwave[i] = sum(Scatter)														// sum up the current share
			endfor
			backwave += Down																	// add lower value
			
			notestr += "\rB = "+num2str(s.TouB)+"\rC = "+num2str(s.TouC)+"\rD = "+num2str(s.TouD)+"\rT0 = "+num2str(s.TouT)+"\rsmoothing = "+num2str(s.TouSmooth)
		break
		case "Tougaard Data":
			Duplicate/Free inwave, CS_Data														// prepare CS data wave
			SetScale/P x 0, abs(delta), CS_Data;	CS_Data = 0									// cross section data needs to be properly scaled from zero
			if (strlen(s.CSdata))
				wave/Z CSWave = $s.CSdata
				if (WaveExists(CSWave))															// check if the wave and the scaling parameter are set
					CS_Data = 0
					P1 = ceil ((leftx(CSWave)-leftx(CS_Data))/deltax(CS_Data))					// start and end points for copying the provided data over
					P2 = trunc((leftx(CSWave)-leftx(CS_Data)+deltax(CSWave)*(numpnts(CSWave)-1))/deltax(CS_Data))		// boundary check
					P2 = P2 > rows-1 ? rows-1 : P2
					P1 = P1 < 0 ? 0 : P1														// protect against negative starting values
					
					if (P1 > P2)
						message += "No useful CS data found."
						backwave = NaN
						break
					endif
					if (P2 < rows - 1)
						message += "CS data range too short."
					endif
					
					CS_Data[P1,P2] = s.CSscale * CSWave(x)
				else
					backwave = NaN
					break
				endif
			endif
			
			Duplicate/Free backwave Scatter;	Scatter = 0										// the scattering share per point
			backwave = 0																		// start with blank background
			for (i = 0; i < rows; i++)
				Scatter[0,rows-i-1] = CS_Data[p] * abs(delta) * (inwave[p+i]-Down)				// calculate the scattering from the current point on
				backwave[i] = sum(Scatter)														// sum up the current share
			endfor
			backwave += Down																	// add lower value
			
			notestr += "\rCS wave = "+s.CSdata
			notestr += "\rCS scaling = "+num2str(s.CSscale)
		break
		default:
			message += "No such function type. Choose from: "+ReplaceString(";",RemoveEnding(kBackgroundTypes),", ")
		break
	Endswitch
	
	DebuggerOptions debugOnError=sav_debug
	
	if (s.followData)
		Duplicate/Free inwave, temp
		if (s.smoothData > 0)
			Smooth limit(s.smoothData, 3, 20), temp
		endif
		backwave[0,P1]		= temp[p]
		backwave[P2,rows-1] = temp[p]
	endif
	
	if (strlen(notestr))
		notestr[0] = "\rsettings:"
	endif
	string noteheader = "function = " + s.FuncType
	noteheader += "\rstart = " + num2str(from) + " " + WaveUnits(inwave,0) + ", Y = " + num2str(Up)
	noteheader += "\rend = " + num2str(to)  + " " + WaveUnits(inwave,0) + ", Y = " + num2str(Down)
	Note/K backwave, noteheader+notestr
	
	return message
end

// *** UI code *** ###############################################################################

function BackgroundSubtractGUI(wave/Z inwave)
	if (!WaveExists(inwave))
		return -1
	endif
	if (WaveType(inwave,1) != 1 || WaveDims(inwave) > 1)
		Abort "This tool works only for numeric 1D waves."
	endif
	//+++++++++++++++++++++++++++++++++ generate working variables ++++++++++++++++++++++++++++++
	string gTitle = "Background Remover (ver. "+num2str(kVersion)+" - "+kVersionDate+"): Background of "
	string gName = UniqueName("InelasticBackDisplay", 6, 1)
	string workF = "BG"+ReplaceString("InelasticBackDisplay",gName,"")+"_"+CleanupName(NameOfWave(inwave),0)[0,25]
	NewDataFolder/O	root:Packages
	NewDataFolder/O	root:Packages:$(kWorkingDir)
	NewDataFolder/O	root:Packages:$(kWorkingDir):$(workF)
	DFREF strg = root:Packages:$(kWorkingDir):$(workF)
	
	String/G strg:InwaveLocation = GetWavesDataFolder(inwave,2)
	Duplicate/O inwave strg:Data_Orig, strg:Data_Net, strg:Data_Back, strg:NegResiduals
	wave/SDFR=strg Data_Orig, Data_Net, Data_Back, NegResiduals
	Data_Net = NaN; Data_Back = NaN; NegResiduals = NaN

	// ++++++++++++++ center the panel in the monitor or the Windows MDI window +++++++++++++++++
	variable pW = 1040, pH = 600, pix = 72/ScreenResolution
	variable scrnL, scrnT, scrnR, scrnB, scrnW, scrnH
	if (CmpStr(IgorInfo(2), "Macintosh") == 0)
		string scrnInfo = StringByKey("SCREEN1", IgorInfo(0))
		scrnInfo = scrnInfo[strsearch(scrnInfo, "RECT=", 0)+5, strlen(scrnInfo)-1]
		sscanf scrnInfo, "%d,%d,%d,%d", scrnL, scrnT, scrnR, scrnB
		scrnW = (scrnR-scrnL) / pix
		scrnH = (scrnB-scrnT) / pix
	elseif (CmpStr(IgorInfo(2), "Windows") == 0)
		GetWindow kwFrameInner, wsize
		scrnW = ScreenResolution == 96 ? (V_right-V_left)/pix : (V_right-V_left)
		scrnH = ScreenResolution == 96 ? (V_bottom-V_top)/pix : (V_bottom-V_top)
	endif
	
	//+++++++++++++++++++++++++++++++++++ generate the panels +++++++++++++++++++++++++++++++++++
	Display/W=((scrnW-pW)*pix/2,(scrnH-pH)*pix/2,(scrnW+pW)*pix/2,(scrnH+pH)*pix/2)/K=2/N=$gName Data_Orig as gTitle + NameOfWave(inwave)
	AppendToGraph/C=(0,0,65280) 		Data_Back												// the calculated background
	AppendToGraph/C=(0,39168,0) 		Data_Net												// the spectrum with subtracted background
	AppendToGraph/C=(0,0,0)/L=Lresid	NegResiduals											// negative values of the net spectrum recalculated for display
	
	ModifyGraph mirror(left)=1, mirror(bottom)=2, standoff=0, ZisZ=1, notation(left)=1, axisOnTop=1, zero(left)=4
	ModifyGraph axisEnab(left)={0.12,1},axisEnab(Lresid)={0,0.08}
	ModifyGraph freePos(Lresid)=0,mode(NegResiduals) = 1, mirror(Lresid)=1,	lblPosMode(Lresid)=1
	SetAxis/A/E=1 Lresid
	SetAxis/A/N=1 left																			// round counts to nice values
	//++++++++++++++++++++++++++++++++++++++++ controls +++++++++++++++++++++++++++++++++++++++++
	SetWindow $gName userdata(workFolder)=workF													// save work folder location
	SetWindow $gName userdata(procVersion)=num2str(kVersion)									// save procedure version
	SetWindow $gName sizeLimit={670*pix, 300*pix, Inf, Inf}										// enforce minimum size => enough to see the main controls (excluding settings)
	
	ControlBar 60																				// add controls
	
	PopUpMenu BackSelect		,pos={10,6}		,size={180,20}		,title="Type:"				,help={"Select the type of background to fit."}
	Setvariable ValA			,pos={10,35}	,size={150,23}		,title="Left Cursor:"		,help={"Set the left cursor position."}
	Setvariable ValB			,pos={185,35}	,size={150,23}		,title="Right Cursor:"		,help={"Set the right cursor position."}
	Setvariable averageCurloc	,pos={205,8}	,size={130,23}		,title="Avg. Range:"		,help={"Percentage of total x range is used to take the average intensity as start and end."}

	Button BSave				,pos={350,6}	,size={125,23}		,title="Export Background"	,help={"Save the current background (blue) and residuals (green) next to the original data without subtracting."}
	CheckBox cPlotOutput		,pos={360,36}	,size={143,23}		,title="Plot after Export"	,help={"Creates a plot of the exported background data together with the input."}
	
	GroupBox ctrlDivider1		,pos={488,8}	,size={1,42}	,frame=0
	Button BMinus				,pos={500,6}	,size={70,20}		,title="Subtract"			,help={"Remove current background from the data."}
	Button BUndo				,pos={500,33}	,size={70,20}		,title="Undo All"			,help={"Revert to source data."}
	Button BStop				,pos={580,6}	,size={70,47}		,title="Overwrite\r& Quit"	,help={"Overwrites the source data and close the window."}
	
	//+++++++++++++++++++++++++++++++ individual background controls ++++++++++++++++++++++++++++
	GroupBox ctrlDivider2		,pos={658,8}	,size={1,42}	,frame=0
	Setvariable VarStepPos		,pos={750,35}	,size={95,23}		,title="Step Position:"		,help={"The position of the background step."}
	Setvariable VarStepSmooth	,pos={910,35}	,size={95,23}		,title="Smoothness:"		,help={"The smoothness of the background step."}
	Setvariable VarSmoothData	,pos={725,35}	,size={95,23}		,title="Smoothing:"			,help={"The smoothness of the data outside the cursor range."}
	
	Setvariable VarPolyDegree	,pos={725,35}	,size={95,23}		,title="Poly Degree:"		,help={"Set the degree of the polynomial fit."}
	
	CheckBox VarFullLinear		,pos={680,36}	,size={143,23}		,title="Calculate over Full Range: "	,help={"Calculate the linear background for the full spectrum width."}
	CheckBox VarFollowData		,pos={845,36}	,size={168,23}		,title="Follow Data outside Range: "	,help={"Makes the background identical with the smoothed original data outside the selected range."}
	
	Setvariable VarTougaardB	,pos={660,35}	,size={95,23}		,title="B:"					,help={"Parameter B (~ height) of the Tougaard universal CS."}
	Setvariable VarTougaardC	,pos={750,35}	,size={95,23}		,title="C:"					,help={"Parameter C (~ position of the maximum) of the Tougaard universal CS."}
	Setvariable VarTougaardD	,pos={840,35}	,size={95,23}		,title="D:"					,help={"Parameter D used for semi-conductors."}
	Setvariable VarTougaardCtoD	,pos={800,8}	,size={85,23}		,title="(D-4xC):"			,help={"The influence of parameter D is removed if D = 4*C."}
	Setvariable VarTougaardT	,pos={930,35}	,size={95,23}		,title="T\B0\M:"			,help={"Parameter T0 (band-gap size) used additionally for insulators."}
	Setvariable VarTouSmooth	,pos={930,8}	,size={95,23}		,title="T\B0\M Smooth:"		,help={"The smoothness of the band-gap step."}
	
	Setvariable VarCSScale		,pos={725,35}	,size={95,23}		,title="CS Scaling:"		,help={"Scales the cross section data to match the spectrum."}
	Setvariable VarExpScale		,pos={725,35}	,size={95,23}		,title="Scale Result:"		,help={"Scales the exponential function to match the spectrum."}
	PopUpMenu VarCSData			,pos={885,35}	,size={140,20}		,title="CS Data:"			,help={"Select a wave with correct cross section data for the material."}
	//+++++++++++++++++++++++++++++++++++ control modifiers ++++++++++++++++++++++++++++++++++++		
	ModifyControlList ControlNameList(gName,";","B*")							,proc=BackRem_ButtonAction
	PopUpMenu BackSelect		,bodywidth=150	,value=kBackgroundTypes			,proc=BackRem_ModeSelect
	PopUpMenu VarCSData			,bodywidth=140	,value=BackRem_CSWaveList()		,proc=BackRem_CSWaveSelect
	
	STRUCT BackRemoveStruct bs
	LoadSettingsFromData(bs, inwave, gName)
	
	variable flip	= sign(DimDelta(Data_Orig,0)) == -1
	variable left	= flip ? pnt2x(Data_Orig,numpnts(Data_Orig)-1) : Dimoffset(Data_Orig,0)		// parameters for cursor placement
	variable right	= flip ? Dimoffset(Data_Orig,0) : pnt2x(Data_Orig,numpnts(Data_Orig)-1)		// for negative delta waves invert everything
	variable inset	= (right-left)/100															// 1% of the wave range
	string unit = WaveUnits(Data_Orig,0)
	unit[0] = SelectString(strlen(unit),""," ")
	
	if (CmpStr(IgorInfo(2), "Macintosh") == 0)
		ModifyControlList ControlNameList(gName,";","*") fsize=10
	endif
	
	Setvariable ValA			,bodywidth=85	,limits={left,right,inset}		,format="%g" + unit	,proc=BackRem_VarChange		,disable=2*(1-flip)
	Setvariable ValB			,bodywidth=85	,limits={left,right,inset}		,format="%g" + unit	,proc=BackRem_VarChange		,disable=2*(flip)
	Setvariable averageCurloc	,bodywidth=65	,limits={0,50,0.1}				,format="%g %"		,proc=BackRem_VarChange		,value=_NUM:bs.avgPercent
	Setvariable VarStepPos		,bodywidth=85	,limits={0.1,50,0.1}			,format="%g" + unit	,proc=BackRem_VarChange		,value=_NUM:bs.stepPos
	Setvariable VarStepSmooth	,bodywidth=65	,limits={0.01,inset*50,inset}	,format="%g" + unit	,proc=BackRem_VarChange		,value=_NUM:bs.stepSmooth
	Setvariable VarSmoothData	,bodywidth=65	,limits={0,500,1}				,format="%g"		,proc=BackRem_VarChange		,value=_NUM:bs.smoothData
	Setvariable VarTougaardB	,bodywidth=65	,limits={1,inf,10}				,format="%g"		,proc=BackRem_VarChange		,value=_NUM:bs.TouB
	Setvariable VarTougaardC	,bodywidth=65	,limits={1,inf,10}				,format="%g"		,proc=BackRem_VarChange		,value=_NUM:bs.TouC
	Setvariable VarTougaardD	,bodywidth=65	,limits={0,inf,10}				,format="%g"		,proc=BackRem_VarChange		,value=_NUM:bs.TouD
	Setvariable VarTougaardCtoD	,bodywidth=55	,limits={0,inf,10}				,format="%g"		,proc=BackRem_VarChange		,value=_NUM:0			,fColor=(30000,30000,30000) ,valueColor=(30000,30000,30000)
	Setvariable VarTougaardT	,bodywidth=65	,limits={0,100,1}				,format="%g"		,proc=BackRem_VarChange		,value=_NUM:bs.TouT
	Setvariable VarTouSmooth	,bodywidth=65	,limits={0.0001,100,0.1}		,format="%g" + unit	,proc=BackRem_VarChange		,value=_NUM:bs.TouSmooth
	Setvariable VarCSScale		,bodywidth=65	,limits={0.001,inf,0.05}		,format="%g"		,proc=BackRem_VarChange		,value=_NUM:bs.CSscale
	Setvariable VarExpScale		,bodywidth=65	,limits={0.05,inf,0.05}			,format="%g"		,proc=BackRem_VarChange		,value=_NUM:bs.expScale
	Setvariable VarPolyDegree	,bodywidth=65	,limits={2,19,1}				,format="%g"		,proc=BackRem_VarChange		,value=_NUM:bs.polyDegree
	CheckBox VarFullLinear														,side=1				,proc=BackRem_CheckAction	,value=bs.fullLinear
	CheckBox VarFollowData														,side=1				,proc=BackRem_CheckAction	,value=bs.followData

	ModifyControlList ControlNameList(gName,";","Var*") ,disable=1								// hide all special controls
	SwitchControlsForMode(bs.FuncType, gName)
	PopUpMenu BackSelect		,popmatch=bs.FuncType
	PopUpMenu VarCSData			,popmatch=bs.CSdata
	
	SetWindow $gName hook(BackCalcAction)=BackRem_WindowHook									// link the function call to the window after the cursors has been set; otherwise eventCode == 7 will trigger)
	
	doBackgroundCalculation(gName)
	
	DoWindow/F $gName
	return 0
end
//################################################################################################

static function/DF getWorkFolder(string gName)
	string folder = ""
	if (strlen(gName))
		folder = GetUserData(gName,"","workFolder")
	endif
	return root:Packages:$(kWorkingDir):$(folder)
end

// *** UI panel hook *** #########################################################################

function BackRem_WindowHook(STRUCT WMWinHookStruct &s)
	DFREF stg = getWorkFolder(s.winName)
	DFREF pkroot = getWorkFolder("")
	DFREF pkg = root:Packages:
	
	int HookTakeover = 0, Objects = 1
	StrSwitch (s.EventName)
		case "cursormoved":
			doBackgroundCalculation(s.winName)
		break
		case "modified":	// probably axis zoom
			HookTakeover = 1
			wave/SDFR=stg Data_Orig
			ControlInfo/W=$s.winName ValA;	variable Left = V_Value
			ControlInfo/W=$s.winName ValB;	variable Right = V_Value
			Cursor/F/W=$s.winName A  Data_Orig Left,  Data_Orig(Left)							// reset the cursors to their previous x value
			Cursor/F/W=$s.winName B  Data_Orig Right, Data_Orig(Right)
		break
		case "killVote":	// window close button
			HookTakeover = 1
			KillWindow $s.winName
			if (CountObjectsDFR(stg, 4) == 0)													// makes sure to not kill the root folder with other open instances
				KillDataFolder/Z stg
			endif
			Objects = CountObjectsDFR(pkroot, 1)+CountObjectsDFR(pkroot, 2)+CountObjectsDFR(pkroot, 3)+CountObjectsDFR(pkroot, 4)
			if (Objects == 0 && DataFolderRefStatus(pkroot) != 0)
				KillDataFolder pkroot
			endif
			Objects = CountObjectsDFR(pkg, 1)+CountObjectsDFR(pkg, 2)+CountObjectsDFR(pkg, 3)+CountObjectsDFR(pkg, 4)
			if (Objects == 0)																	// clean up Packages folder if empty
				KillDataFolder pkg
			endif
		break
	EndSwitch
	return HookTakeover
end

// *** load settings *** #########################################################################

static function LoadSettingsFromData(STRUCT BackRemoveStruct &bs, wave inwave, string gName)
	wave/Z backwave = $(GetWavesDataFolder(inwave,1)+PossiblyQuoteName(NameOfWave(inwave)+"_bck"))
	
	//+++++++++++++++++++++++++++++++++ initialize parameters ++++++++++++++++++++++++++++++++++++
	BackRemoveInitialize(bs)
	variable flip	= sign(DimDelta(inwave,0)) == -1
	variable left	= flip ? pnt2x(inwave,numpnts(inwave)-1) : Dimoffset(inwave,0)				// parameters for cursor placement
	variable right	= flip ? Dimoffset(inwave,0) : pnt2x(inwave,numpnts(inwave)-1)				// for negative delta waves invert everything
	variable inset	= (right-left)/100															// 1% of the wave range
	WaveStats/Q/R = (left+10*inset,right-10*inset) inwave;		bs.stepPos = V_maxloc			// initial step position for step function
	//+++++++++++++++++++++++++++++++++ load previous settings +++++++++++++++++++++++++++++++++++
	string settings = ""
	if (WaveExists(backwave))
		settings = ReplaceString("\r",ReplaceString(" = ",note(backwave),"="),";")
	endif
	string funcType = StringByKey("function", settings, "=")
	if (strlen(funcType))
		bs.FuncType = funcType
	endif

	variable startCur = str2num(StringByKey("start", settings, "="))
	variable stopCur  = str2num(StringByKey("end", settings, "="))
		
	if (abs(startCur-left) < 10^-3*abs(DimDelta(inwave,0)))										// start point may be slightly out-of-range because of rounding errors
		startCur = left
	endif
	if (abs(stopCur-right) < 10^-3*abs(DimDelta(inwave,0)))
		stopCur = right
	endif
	
	DoUpdate																					// update graph before placing cursors
	if (numtype(startCur) == 0 && (startCur <= right && startCur >= left))
		Cursor/F/H=2/N=1/W=$gName A  $("Data_Orig") startCur,  inwave(startCur)
	else
		Cursor/F/H=2/N=1/W=$gName A  $("Data_Orig") (left+10*inset),  inwave(left+10*inset)
	endif
	if (numtype(stopCur) == 0 && (stopCur <= right && stopCur >= left))
		Cursor/F/H=2/N=1/W=$gName B  $("Data_Orig") stopCur, inwave(stopCur)
	else
		Cursor/F/H=2/N=1/W=$gName B  $("Data_Orig") (right-10*inset), inwave(right-10*inset)
	endif
	
	string strVal = ""
	variable numVal = NaN
	strVal = StringByKey("full line", settings, "=");				bs.fullLinear = strlen(strVal) > 0 ? StringMatch(strVal, "yes") : bs.fullLinear
	numVal = str2num(StringByKey("step position", settings, "="));	bs.stepPos = numtype(numVal) == 0 ? numVal : bs.stepPos
	numVal = str2num(StringByKey("step smoothing", settings, "="));	bs.stepSmooth = numtype(numVal) == 0 ? numVal : bs.stepSmooth
	numVal = str2num(StringByKey("exp. scale", settings, "="));		bs.expScale = numtype(numVal) == 0 ? numVal : bs.expScale
	numVal = str2num(StringByKey("poly. degree", settings, "="));	bs.polyDegree = numtype(numVal) == 0 ? numVal : bs.polyDegree
	numVal = str2num(StringByKey("B", settings, "="));				bs.TouB = numtype(numVal) == 0 ? numVal : bs.TouB
	numVal = str2num(StringByKey("C", settings, "="));				bs.TouC = numtype(numVal) == 0 ? numVal : bs.TouC
	numVal = str2num(StringByKey("D", settings, "="));				bs.TouD = numtype(numVal) == 0 ? numVal : bs.TouD
	numVal = str2num(StringByKey("T0", settings, "="));				bs.TouT = numtype(numVal) == 0 ? numVal : bs.TouT
	numVal = str2num(StringByKey("smoothing", settings, "="));		bs.TouSmooth  = numtype(numVal) == 0 && CmpStr(bs.FuncType,"Tougaard") == 0 ? numVal : bs.TouSmooth
																	bs.smoothData = numtype(numVal) == 0 && CmpStr(bs.FuncType,"Tougaard") != 0 ? numVal : bs.smoothData
	numVal = str2num(StringByKey("CS scaling", settings, "="));		bs.CSscale = numtype(numVal) == 0 ? numVal : bs.CSscale
	strVal = StringByKey("CS wave", settings, "=");					bs.CSdata = SelectString(strlen(strVal), bs.CSdata, strVal)
	strVal = StringByKey("follow data", settings, "=");				bs.followData = strlen(strVal) > 0 ? StringMatch(strVal, "yes") : bs.followData
	return 0
end

// *** UI control elements code *** #############################################################

function BackRem_ModeSelect(STRUCT WMPopupAction &s) : PopupMenuControl
	if (s.eventCode == 2)
		SwitchControlsForMode(s.popStr, s.win)
		doBackgroundCalculation(s.win)
	endif
	return 0
end

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static function SwitchControlsForMode(string mode, string gName)
	wave/SDFR=getWorkFolder(gName)/Z Data_Orig, Data_Orig_CS
	if (WaveExists(Data_Orig_CS))																// clean up possible CS wave from parametric Tougaard
		KillWaves/Z Data_Orig_CS
	endif

	variable onlyOneCursor = 0, chkver = str2num(GetUserData(gName, "", "procVersion"))			// make sure feature changes only appear in newer versions
	chkver = numtype(chkver) != 0 ? 0 : chkver
	string list = ""
	Strswitch(mode)
		case "Step":
			list = "VarStepPos;VarStepSmooth;"
		break
		case "Linear":
			list = "VarFullLinear;"
		break
		case "Double Exponential":
		case "Exponential":
			list = SelectString(chkver>0,"","VarFollowData;")+"VarExpScale;"
		break
		case "Polynomial":
			list = SelectString(chkver>0,"","VarFollowData;")+"VarPolyDegree;"
		break
		case "Shirley":
		case "Total Sum":
			list = "VarFollowData;VarSmoothData;"
		break
		case "Constant":
			onlyOneCursor = 1
		break
		case "Tougaard":
			onlyOneCursor = 1
			list = ControlNameList(gName,";","VarTou*")
		break
		case "Tougaard Data":
			onlyOneCursor = 1
			list = "VarCSScale;VarCSData;"
		break
	Endswitch
	
	ModifyControlList ControlNameList(gName,";","Var*") ,win=$gName ,disable=1					// hide all controls
	ModifyControlList list+"ValA;ValB;" ,win=$gName ,disable=0									// enable relevant controls
	
	if (onlyOneCursor)
		if (sign(DimDelta(Data_Orig,0)) == -1)
			Setvariable ValB ,win=$gName ,disable=2												// disable unused cursor (cosmetic change)
		else
			Setvariable ValA ,win=$gName ,disable=2
		endif
	endif
	return 0
end


//################################################################################################

function BackRem_CheckAction (STRUCT WMCheckboxAction &s) : CheckBoxControl
	if (s.eventCode == 2)
		doBackgroundCalculation(s.win)
	endif
	return 0
end

//################################################################################################

function BackRem_CSWaveSelect(STRUCT WMPopupAction &s) : PopupMenuControl
	if (s.eventCode == 2)
		doBackgroundCalculation(s.win)
	endif
	return 0
end
//+++++++++++++++++++++++++++++++++++++ popup help functions +++++++++++++++++++++++++++++++++++++
function/S BackRem_CSWaveList()
	return "none;" + WaveList("*",";", "DIMS:1")
end

//################################################################################################

function BackRem_ButtonAction(STRUCT WMButtonAction &s) : ButtonControl
	if (s.eventCode != 2)
		return 0
	endif
	
	DFREF stg = getWorkFolder(s.win)
	wave/SDFR=stg Data_Orig, Data_Back, Data_Net
	SVAR inwaveLoc = stg:InwaveLocation
	wave inwave = $inwaveLoc
	DFREF inwaveDF = GetWavesDataFolderDFR(inwave)
	
	string NewNote = "Background was removed!\r"
	string AddNote = note(Data_Back)
	string inwName = NameOfWave(inwave)
	
	if (strlen(note(inwave)))
		NewNote[0] = "\r"																		// add a bit of space
	endif
	
	Strswitch(s.ctrlName)
		case "BMinus":
			Data_Orig -= Data_Back
			Note Data_Orig, NewNote+AddNote														// write the latest background info into the wave note
			Note Data_Net, NewNote+AddNote
			doBackgroundCalculation(s.win)
		break
		case "BUndo":
			Duplicate/O inwave stg:Data_Orig
			doBackgroundCalculation(s.win)
		break
		case "BSave":
			Duplicate/O stg:Data_Net  inwaveDF:$(inwName+"_net")
			Duplicate/O stg:Data_Back inwaveDF:$(inwName+"_bck")
			Note inwaveDF:$(inwName+"_net"), NewNote+AddNote
			wave/Z CSwave = stg:Data_Orig_CS
			if(WaveExists(CSwave))
				Duplicate/O CSwave inwaveDF:$(inwName+"_scs")									// if available, copy generated Tougaard cs wave
				Note/K inwaveDF:$(inwName+"_scs"), AddNote
			endif
			
			ControlInfo/W=$s.win cPlotOutput
			if (V_Flag == 2 && V_Value == 1)													// additionally plot the output
				wave back = inwaveDF:$(inwName+"_bck")
				wave wnet = inwaveDF:$(inwName+"_net")
				string settings = ReplaceString("\r",ReplaceString(" = ",AddNote,"="),";")
				
				Display/K=1/W=(200,50,700,300) inwave as "Background for " + inwName			// display the results
				AppendToGraph/C=(0,0,65280) back
				AppendToGraph/C=(0,39200,0) wnet
				ModifyGraph mirror(left)=1, mirror(bottom)=2, minor=1, standoff=0, ZisZ=1, notation(left)=1
				
				Legend/C/N=text0/B=1/F=0/A=LT													// add a legend with the settings
				AppendText/N=text0 StringByKey("function", settings, "=") +" background"
				variable split = strsearch(AddNote,"settings:",0)
				if (split > -1)
					settings = AddNote[split+strlen("settings:")+1,inf]
					AppendText/N=text0/NOCR ":"
					AppendText/N=text0 ReplaceString("\r",settings,", ")
				endif
			endif
		break
		case "BStop":
			if (!EqualWaves(Data_Orig, inwave,-1))												// compare if the wave was changed
				Duplicate/O stg:Data_Orig inwave
				Note inwave, NewNote+AddNote
			endif
			KillWindow $s.win
		break
	Endswitch
	return 0
end

//################################################################################################

function BackRem_VarChange(STRUCT WMSetVariableAction &s) : SetVariableControl
	if (s.eventCode == 1 || s.eventCode == 2)
		wave/SDFR=getWorkFolder(s.win) Data_Orig
		string CsrName = ""
		Strswitch(s.ctrlName)
			case "ValA":
				CsrName = SelectString(hcsr(A,s.win) > hcsr(B,s.win) , "A", "B")
			break
			case "ValB":
				CsrName = SelectString(hcsr(A,s.win) > hcsr(B,s.win) , "B", "A")
			break
			default:
				if (CmpStr(s.ctrlName,"VarTougaardCtoD") == 0)
					ControlInfo/W=$s.win VarTougaardC
					Setvariable VarTougaardD ,win=$s.win ,value=_NUM:(4*V_Value+s.dval)
				endif
				doBackgroundCalculation(s.win)
		EndSwitch
		If (strlen(CsrName))
			Cursor/F/W=$s.win $CsrName  Data_Orig s.dval, Data_Orig(s.dval)
		endif
	endif
	return 0
end

// *** UI core worker *** ########################################################################

static function doBackgroundCalculation(string gName)
	variable mainCur, subCur
	STRUCT BackRemoveStruct bs
	
	GetCursorLocation(mainCur,subCur, gName)
	UpdateCursorControls(mainCur, subCur, gName)
	ReadAllPanelSettings(bs, gName)
	
	wave/SDFR=getWorkFolder(gName) Data_Orig, Data_Net, Data_Back, NegResiduals
	string status = GenerateBackground(bs, Data_Orig, Data_Back, from=mainCur, to=subCur)
	Data_Net = Data_Orig - Data_Back															// subtract background
	NegResiduals = (Data_Net < 0) ? abs(Data_Net*100/Data_Orig) : NaN							// calculate residuals below zero
	
	string GraphText = "\Z10\\s(Data_Orig) current spectrum"
	GraphText += "\r\\s(Data_Back) background"
	GraphText += "\r\\s(Data_Net) net spectrum"
	GraphText += "\r\\s(NegResiduals) negative values (%)"
	GraphText += "\K(65280,0,0)\r" + status
	int i
	for (i = 2; i < bs.polyDegree; i++)
		GraphText = ReplaceString("*x^"+num2str(i),GraphText,"x\S"+num2str(i)+"\M\Z10" + SelectString(mod(i,5) == 0,"","\r"))	// poly background: stylize exponent; add a line-break once in a while
	endfor
	GraphText = ReplaceString("*x",GraphText,"x")
	Legend/C/N=GraphInfoBox/J/B=1/F=0/A=LT/W=$gName GraphText
	return 0
end

//################################################################################################

static function UpdateCursorControls(variable mainCur, variable subCur, string gName)			// write cursor locations into the GUI controls
	variable low  = mainCur < subCur ? mainCur : subCur
	variable high = mainCur > subCur ? mainCur : subCur
	Setvariable VarStepPos	,win=$gName ,limits={low,high,abs(high-low)*0.01}					// set limits of step control
	Setvariable ValA 		,win=$gName ,value=_NUM:low
	Setvariable ValB 		,win=$gName ,value=_NUM:high
	return 0
end

//################################################################################################

static function GetCursorLocation(variable &mainCur, variable &subCur, string gName)			// updates the current cursor location and gets the position
	wave/SDFR=getWorkFolder(gName) Data_Orig
	mainCur	= hcsr(A,gName)
	subCur	= hcsr(B,gName)
	variable flip = sign(DimDelta(Data_Orig,0))
	if (flip*mainCur > flip*subCur)
		mainCur	= hcsr(B,gName)
		subCur	= hcsr(A,gName)
	endif
	variable lbound = DimOffset(Data_Orig,0)
	variable hbound = lbound + (DimSize(Data_Orig,0)-1)*DimDelta(Data_Orig,0)
	mainCur	= flip*mainCur < flip*lbound ? lbound : mainCur
	subCur	= flip*subCur  > flip*hbound ? hbound : subCur
	return 0
end

//################################################################################################

static function ReadAllPanelSettings(STRUCT BackRemoveStruct &info, string gName)
	ControlInfo/W=$gName ValA;				variable low  = V_Value
	ControlInfo/W=$gName ValB;				variable high = V_Value
	ControlInfo/W=$gName BackSelect;		info.FuncType = S_Value
	ControlInfo/W=$gName averageCurloc;		info.avgPercent = V_Value
	ControlInfo/W=$gName VarSmoothData;		info.smoothData = V_Value
	ControlInfo/W=$gName VarStepPos;		info.stepPos = limit(V_Value, low, high)
	ControlInfo/W=$gName VarStepSmooth;		info.stepSmooth = V_Value
	ControlInfo/W=$gName VarTougaardB;		info.TouB = V_Value
	ControlInfo/W=$gName VarTougaardC;		info.TouC = V_Value
	ControlInfo/W=$gName VarTougaardD;		info.TouD = V_Value
	ControlInfo/W=$gName VarTougaardT;		info.TouT = V_Value
	ControlInfo/W=$gName VarTouSmooth;		info.TouSmooth = abs(V_flag) == 5 ? V_Value : 0.5
	ControlInfo/W=$gName VarCSScale;		info.CSscale = V_Value
	ControlInfo/W=$gName VarExpScale;		info.expScale = V_Value
	ControlInfo/W=$gName VarPolyDegree;		info.polyDegree = limit(V_Value+1,3,20)
	ControlInfo/W=$gName VarCSData;			info.CSdata = SelectString(!V_Disable, "", S_Value)	// get the wave to use for Tougaard calculation
	ControlInfo/W=$gName VarFollowData;		info.followData = V_Value && !V_Disable				// visible and active
	ControlInfo/W=$gName VarFullLinear;		info.fullLinear= V_Value
	ControlInfo/W=$gName VarTougaardCtoD
	if(abs(V_flag) == 5)
		Setvariable VarTougaardCtoD ,win=$gName ,value=_NUM:(info.TouD-4*info.TouC) ,limits={-4*info.TouC,inf,10}
	endif
	return 0
end